﻿Imports System.Drawing.Drawing2D

Public Class RegionInfo
    Public TheRegion As Region
    Public Count As Integer

    ' Make the RegionInfo for a circle.
    Public Sub New(ByVal new_count As Integer, ByVal center As PointF, ByVal radius As Single)
        Count = new_count

        ' Make the Region.
        Using circle_path As New GraphicsPath()
            circle_path.AddEllipse( _
                center.X - radius, center.Y - radius, _
                2 * radius, 2 * radius)
            TheRegion = New Region(circle_path)
        End Using
    End Sub

    ' Make a RegionInfo for a Region.
    Public Sub New(ByVal new_count As Integer, ByVal new_region As Region)
        ' Save the count and region.
        Count = new_count
        TheRegion = new_region
    End Sub

    ' For each RegionInfo "other" in the list, intersect
    ' "this" RegionInfo with other and make up to 3
    ' new RegionInfos:
    '
    '      this - other
    '      other - this
    '      this intersect other
    '
    ' We assume that we are not yet on the list.
    Public Sub MakeIntersections(ByVal gr As Graphics, ByVal region_infos As List(Of RegionInfo))
        ' Make a list to hold the new RegionInfos.
        Dim new_regioninfos As New List(Of RegionInfo)()

        ' Intersect with the RegionInfos on the list.
        For Each other As RegionInfo In region_infos
            ' Find this intersect other.
            Dim intersection As Region = Me.TheRegion.Clone()
            intersection.Intersect(other.TheRegion)

            ' See if the intersection is empty.
            If (intersection.IsEmpty(gr)) Then
                ' The intersection is empty. 
                ' Keep this and other as they are.
                intersection.Dispose()
            Else
                ' The intersection is not empty.
                ' Replace other with other - me.
                other.TheRegion.Xor(intersection)

                ' Replace this with this - other.
                Me.TheRegion.Xor(intersection)

                ' Add the intersection to the new list.
                new_regioninfos.Add( _
                    New RegionInfo(Me.Count + other.Count, intersection))
            End If
        Next other

        ' Add the new RegionInfos to the original list.
        For Each new_regioninfo As RegionInfo In new_regioninfos
            region_infos.Add(new_regioninfo)
        Next new_regioninfo
    End Sub

    ' Return the centroid of the region.
    Private Function RegionCentroid(ByVal the_region As Region, ByVal transform As Matrix) As PointF
        Dim cx As Single = 0
        Dim cy As Single = 0
        Dim total_weight As Single = 0
        For Each rect As RectangleF In the_region.GetRegionScans(transform)
            Dim rect_weight As Single = rect.Width * rect.Height
            cx += rect_weight * (rect.Left + rect.Width / 2.0F)
            cy += rect_weight * (rect.Top + rect.Height / 2.0F)
            total_weight += rect_weight
        Next rect

        Return New PointF(cx / total_weight, cy / total_weight)
    End Function

    ' Draw the Region with the count at its centroid.
    Public Sub Draw(ByVal gr As Graphics, ByVal the_font As Font, ByVal max_count As Integer)
        ' See what color to use.
        Dim r As Integer = CInt(Int(255 * Count / max_count))
        Dim fill_color As Color = Color.FromArgb(255, 255, r, 0)

        ' Fill the Region.
        Using fill_brush As New SolidBrush(fill_color)
            gr.FillRegion(fill_brush, TheRegion)
        End Using

        ' Draw the count.
        Using string_format As New StringFormat()
            string_format.Alignment = StringAlignment.Center
            string_format.LineAlignment = StringAlignment.Center
            gr.DrawString(Count.ToString(), the_font, Brushes.Black, _
                RegionCentroid(TheRegion, gr.Transform), string_format)
        End Using
    End Sub
End Class
